#!/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Author:      thepoy
# @Email:       thepoy@163.com
# @File Name:   __init__.py
# @Created At:  2021-06-18 11:08:30
# @Modified At: 2023-02-21 13:00:13
# @Modified By: thepoy

import os

from pathlib import Path

BASE_DIR = Path(os.path.dirname(os.path.abspath(__file__)))
IMAGES_DIR = BASE_DIR / "images"

IMAGES = [
    IMAGES_DIR / "1.png",
    IMAGES_DIR / "2.jpeg",
    IMAGES_DIR / "3.jpeg",
]



#!/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Author:      thepoy
# @Email:       thepoy@163.com
# @File Name:   __main__.py
# @Created At:  2021-02-08 15:47:06
# @Modified At: 2023-02-21 12:43:38
# @Modified By: thepoy

import up2b

up2b.run_main()



#!/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Author:      thepoy
# @Email:       thepoy@163.com
# @File Name:   test_upload.py
# @Created At:  2022-09-02 09:17:46
# @Modified At: 2023-02-21 14:05:40
# @Modified By: thepoy

import sys
import os
import json
import mimetypes
import time
import requests
import pytest

from typing import Union
from multiprocessing import Process
from pathlib import Path

from tests import IMAGES
from up2b.up2b_lib.errors import OverSizeError
from up2b.up2b_lib.up2b_api import Base
from up2b.up2b_lib.custom_types import (
    ImageBedType,
    ImageType,
    ImageStream,
    ImagePath,
    UploadErrorResponse,
)
from up2b.up2b_lib.constants import ImageBedCode
from tests.server import HOST, run_server, PORT


class ImageBed(Base):
    image_bed_type = ImageBedType.common
    image_bed_code = ImageBedCode.IMGTG
    max_size = 50 * 1024
    base_url = f"http://{HOST}:{PORT}/"

    def __upload(self, image: ImageType, retries=0) -> Union[str, UploadErrorResponse]:
        image = self._compress_image(image)

        if isinstance(image, Path):
            image = self._add_watermark(image)

        filename_with_suffix = os.path.basename(str(image))
        filename_without_suffix, suffix = os.path.splitext(filename_with_suffix)
        if isinstance(image, Path):
            if suffix.lower() == ".apng":
                suffix = ".png"
            filename = filename_without_suffix + suffix
        else:
            filename = filename_with_suffix + "." + image.mime_type

        mime_type = (
            mimetypes.guess_type(image)[0]
            if isinstance(image, Path)
            else image.mime_type
        )

        if isinstance(image, Path):
            with open(image, "rb") as fb:
                img_buffer = fb.read()
        else:
            img_buffer = image.stream

        files = {
            "source": (filename, img_buffer, mime_type),
        }

        resp = requests.post(
            self.base_url, files=files, timeout=self.timeout  # type: ignore
        )
        resp.encoding = "utf-8"

        try:
            json_resp = resp.json()
        except json.decoder.JSONDecodeError:
            return UploadErrorResponse(resp.status_code, resp.text, str(image))

        return json_resp["url"]

    def upload_image(self, image_path: ImagePath) -> Union[str, UploadErrorResponse]:
        image_path = self._add_watermark(image_path)

        if self.auto_compress:
            # 输入的是路径，返回的也必然是路径，忽略 pyright 的错误提示
            image_path = self._compress_image(image_path)  # type: ignore

        return self.__upload(image_path)

    def upload_image_stream(
        self, image: ImageStream
    ) -> Union[str, UploadErrorResponse]:
        if self.auto_compress:
            new_image = self._compress_image(image)
        else:
            new_image = image

        return self.__upload(new_image)

    def delete_image(self):
        pass

    def delete_images(self):
        pass

    def get_all_images(self):
        pass

    @property
    def description(self):
        return ""


class TestUpload:
    ib = ImageBed()

    def test_upload(self):
        t = Process(target=run_server)
        t.daemon = True
        t.start()

        if sys.platform == "darwin":
            # macOS 多进程启动服务器有点慢，需要 sleep
            time.sleep(10)

        resp = self.ib.upload_image(IMAGES[0])
        assert resp == f"http://{HOST}:{PORT}/images/1.png"

        t.terminate()

    def test_check_images(self):
        with pytest.raises(OverSizeError):
            self.ib._check_images_valid(IMAGES)



#!/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Author:      thepoy
# @Email:       thepoy@163.com
# @File Name:   test_utils.py
# @Created At:  2023-02-07 10:48:08
# @Modified At: 2023-03-09 10:11:44
# @Modified By: thepoy

from pathlib import Path
from tests import IMAGES
from up2b.up2b_lib import utils
from up2b.up2b_lib.constants import CACHE_PATH
from up2b.up2b_lib.custom_types import DownloadErrorResponse


class TestUtils:
    def test_is_url(self):
        yes = utils.is_url(str(IMAGES[0]))
        assert yes == False

        yes = utils.is_url(
            "https://raw.githubusercontent.com/thep0y/image-bed/main/test/test.jpg"
        )
        assert yes == True

    def test_check_path(self):
        path = utils.check_path(IMAGES[0])
        assert IMAGES[0] == path

        path = utils.check_path(
            "https://raw.githubusercontent.com/thep0y/image-bed/main/test/1647607453268.jpg"
        )
        assert CACHE_PATH / "1647607453268.jpg" == path

        path = utils.check_path(
            "https://raw.githubusercontent.com/thep0y/image-bed/main/test/test.jpg"
        )
        assert isinstance(path, DownloadErrorResponse)



#!/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Author:      thepoy
# @Email:       thepoy@163.com
# @File Name:   cache.py
# @Created At:  2023-02-28 22:16:22
# @Modified At: 2023-03-06 20:34:24
# @Modified By: thepoy

import sqlite3
import hashlib

from up2b.up2b_lib.constants import CACHE_DATABASE, PYTHON_VERSION

if PYTHON_VERSION >= (3, 9):
    from functools import cache
else:
    from functools import lru_cache

    cache = lru_cache(maxsize=None)

from typing import Any, Optional, Tuple
from pathlib import Path
from up2b.up2b_lib.log import child_logger


logger = child_logger(__name__)


@cache
def file_md5(filepath: Path) -> str:
    h = hashlib.md5()

    with filepath.open("rb") as f:
        while True:
            data = f.read(1024)
            if not data:
                break

            h.update(data)

    return h.hexdigest()


class Cache:
    def __init__(self) -> None:
        self.conn = sqlite3.connect(CACHE_DATABASE)

        self.create_table()

    def create_table(self):
        c = self.conn.cursor()
        sql = """
        CREATE TABLE IF NOT EXISTS cache (
            id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
            hash CHAR(128) NOT NULL,
            image_bed TEXT NOT NULL,
            url TEXT NOT NULL UNIQUE
        );
        """
        c.execute(sql)
        sql = """
            CREATE UNIQUE INDEX IF NOT EXISTS idx_hash_ib
                ON cache (hash, image_bed);
        """
        c.execute(sql)
        self.commit()

    @cache
    def is_exists(self, md5: str, image_bed: Optional[str] = None) -> Optional[str]:
        if image_bed:
            sql = """
                SELECT url FROM cache
                WHERE hash = ? AND image_bed = ?;
            """

            c = self.execute(sql, md5, image_bed)
        else:
            sql = """
                SELECT url FROM cache WHERE hash = ?;
            """

            c = self.execute(sql, md5)

        result = c.fetchone()
        if not result:
            return None

        return result[0]

    @cache
    def check_cache_of_image_bed(
        self, filepath: Path, image_bed: str
    ) -> Tuple[str, str, bool]:
        """精准查询指定图床中是否缓存过图片

        :param filepath: 图片真实路径或缓存路径
        :type filepath: Path
        :param image_bed: 图床名
        :type image_bed: str
        :returns: 缓存的图片链接或文件 md5
        :rtype: Tuple[str,str, bool]
        """

        md5 = file_md5(filepath)

        logger.debug("the md5 of the file is calculated", file=filepath, md5=md5)

        url = self.is_exists(md5, image_bed)
        if url:
            return (url, md5, True)

        return ("", md5, False)

    @cache
    def chech_cache(self, filepath: Path) -> Tuple[str, str, bool]:
        """模糊查询图片缓存。

        不论任何在图床中上传过图片，只要查询到即返回已上传的图片链接。
        :param filepath: 图片真实路径或缓存路径
        :type filepath: Path
        :returns: 缓存的图片链接或文件 md5
        :rtype: Tuple[str, str, bool]
        """

        md5 = file_md5(filepath)

        logger.debug("the md5 of the file is calculated", file=filepath, md5=md5)

        url = self.is_exists(md5)
        if url:
            return (url, md5, True)

        return ("", md5, False)

    def save(self, md5: str, image_bed: str, url: str, force: bool = False):
        exists = self.is_exists(md5, image_bed)

        if exists:
            if force:
                sql = """
                    UPDATE cache SET url = ? WHERE hash = ? AND image_bed = ?;
                """

                return self.execute(sql, url, md5, image_bed)

            logger.warning("图片已存在且未开启强制更新，图片链接不会保存")
        else:
            sql = """
                INSERT INTO cache (url, hash, image_bed) VALUES (?, ?, ?);
            """

            return self.execute(sql, url, md5, image_bed)

    def execute(self, sql: str, *params: Any):
        c = self.conn.cursor()
        return c.execute(sql, params)

    def commit(self):
        self.conn.commit()

    def close(self):
        self.conn.close()

    def __del__(self):
        self.commit()
        self.close()



#!/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Author:      thepoy
# @Email:       thepoy@163.com
# @File Name:   custom_types.py
# @Created At:  2021-02-09 11:27:21
# @Modified At: 2023-02-21 12:41:21
# @Modified By: thepoy

from enum import IntEnum
from typing import Any, Dict, List, Union
from dataclasses import dataclass, asdict
from pathlib import Path


class ImageBedType(IntEnum):
    common = 1
    git = 2


@dataclass
class ImageStream:
    filename: str
    stream: bytes
    mime_type: str

    def __repr__(self) -> str:
        return self.filename


ImagePath = Path
AuthInfo = Dict[str, Any]
AuthData = Dict[str, AuthInfo]
WaterMarkConfig = Dict[str, int]
ConfigFile = Dict[str, Union[int, AuthData, WaterMarkConfig]]
DeletedResponse = Dict[str, Union[bool, str]]
ImageType = Union[ImagePath, ImageStream]
Images = List[ImageType]


@dataclass
class BaseResponse:
    def to_dict(self) -> Dict[str, Any]:
        return asdict(self)


@dataclass
class ErrorResponse(BaseResponse):
    status_code: int
    error: Union[str, Dict[str, Any]]


@dataclass
class DownloadErrorResponse(ErrorResponse):
    pass


@dataclass
class UploadErrorResponse(ErrorResponse):
    image_path: str


@dataclass
class GitGetAllImagesResponse(BaseResponse):
    url: str
    sha: str
    delete_url: str


@dataclass
class SMMSResponse(BaseResponse):
    url: str
    delete_url: str
    width: int
    height: int


@dataclass
class CodingResponse(BaseResponse):
    url: str
    filename: str


@dataclass
class ImgtuResponse(BaseResponse):
    id: str
    url: str
    display_url: str
    width: int
    height: int



#!/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Author:      thepoy
# @Email:       thepoy@163.com
# @File Name:   log.py
# @Created At:  2023-02-07 12:13:41
# @Modified At: 2023-02-21 12:40:41
# @Modified By: thepoy

from colorful_logger import get_logger, child_logger as cl
from colorful_logger.logger import is_debug
from up2b.up2b_lib.constants import CONFIG_FOLDER_PATH

log_file_path = None
show = True

if is_debug():
    log_file_path = CONFIG_FOLDER_PATH / "up2b.log"
    show = True


def child_logger(name: str):
    return cl(name, logger)


logger = get_logger(
    "up2b",
    show=show,
    file_path=log_file_path,
    add_file_path=False,
    disable_line_number_filter=True,
)



#!/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Author:      thepoy
# @Email:       thepoy@163.com
# @File Name:   i18n.py
# @Created At:  2022-01-09 11:17:23
# @Modified At: 2023-02-27 22:47:19
# @Modified By: thepoy

import json
import os

from typing import Dict
from up2b.up2b_lib.constants import CONFIG_FOLDER_PATH
from up2b.up2b_lib.utils import get_default_language
from up2b.up2b_lib.log import child_logger

logger = child_logger(__name__)

_i18n_folder = CONFIG_FOLDER_PATH / "i18n"

LangType = str
English = str
Translation = str
Locale = Dict[English, Translation]

_i18ns: Dict[LangType, Locale] = {
    "en_US": {
        "A package that can upload images to the image bed.": "A package that can upload images to the image bed.",
        "allow automatic image compression": "allow automatic image compression",
        "whether to add text watermark to the images to be uploaded": "whether to add text watermark to the images to be uploaded",
        "show the image bed in use": "show the image bed in use",
        "list all configured image beds": "list all configured image beds",
        "choose the image bed you want to use and exit": "choose the image bed you want to use and exit",
        "save the user authentication token after successful login. You must enter the username and password after `-l` or `--login`": "save the user authentication token after successful login. You must enter the username and password after `-l` or `--login`",
        "save the authentication information of the git website, such as github": "save the authentication information of the git website, such as github",
        "configure the text watermark": "configure the text watermark",
        "upload only one picture": "upload only one picture",
        "upload multiple pictures, the maximum is 10 pictures, use spaces to separate each image path": "upload multiple pictures, the maximum is 10 pictures, use spaces to separate each image path",
        "save the authentication information of coding": "save the authentication information of coding",
    },
    "zh_CN": {
        "A package that can upload images to the image bed.": "一个能将本地图片压缩、加水印或原图上传到图床的包",
        "allow automatic image compression": "允许自动压缩图片。开启此项后，超过图床限制大小的图片将会自动压缩到限制大小后再上传",
        "whether to add text watermark to the images to be uploaded": "对要上传的图片添加文字水印",
        "show the image bed in use": "显示正在使用的图床",
        "list all configured image beds": "列出所有已配置的图床",
        "choose the image bed you want to use and exit": "选择要使用的图床",
        "save the user authentication token after successful login. You must enter the username and password after `-l` or `--login`": "保存认证信息。你需要在`-l`或`--login`后输入账名和密码",
        "save the authentication information of the git website, such as github": "保存 git 类型图床的认证信息，如 github",
        "configure the text watermark": "配置要添加的文字水印",
        "upload only one picture": "上传一张图片",
        "upload multiple pictures, the maximum is 10 pictures, use spaces to separate each image path": "上传多张图片。多张图片路径之间以空格分隔，最多能上传 10 张",
        "save the authentication information of coding": "保存 Coding 的认证信息",
    },
}


def read_i18n() -> Locale:
    if not _i18n_folder.exists():
        _i18n_folder.mkdir()

        logger.debug("the language used: [ en_US ]")

        return _i18ns["en_US"]

    lang = get_default_language()
    locale = _i18ns.get(lang)
    if locale:
        logger.debug("the language used: [ %s ]", lang)
        return locale

    translation_file = _i18n_folder / (lang + ".json")
    if not os.path.exists(translation_file):
        logger.info(
            "the language used is [ %s ], but no translation file for [ %s ] was found, so use [ en_US ]",
            lang,
            lang,
        )
        return _i18ns["en_US"]

    logger.debug("translation file [ %s ] will be used", translation_file)

    with translation_file.open() as f:
        return json.loads(f.read())



#!/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Author:      thepoy
# @Email:       thepoy@163.com
# @File Name:   imgtu.py
# @Created At:  2021-02-13 09:04:37
# @Modified At: 2023-03-04 21:38:13
# @Modified By: thepoy

import os
from pathlib import Path
import re
import time
import json
import mimetypes
import requests

from urllib import parse
from typing import Dict, List, Optional, Tuple, Union
from up2b.up2b_lib.custom_types import (
    ErrorResponse,
    ImageBedType,
    ImageType,
    ImageStream,
    ImagePath,
    ImgtuResponse,
    UploadErrorResponse,
)
from up2b.up2b_lib.errors import MissingAuth
from up2b.up2b_lib.up2b_api import Base
from up2b.up2b_lib.constants import IMAGE_BEDS_NAME, ImageBedCode
from up2b.up2b_lib.log import child_logger

logger = child_logger(__name__)


class Imgtu(Base):
    image_bed_type = ImageBedType.common
    image_bed_code = ImageBedCode.IMGTU
    max_size = 10 * 1024 * 1024
    base_url = "https://imgse.com/"
    __headers = {
        "Accept": "application/json",
        "User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:85.0) Gecko/20100101 Firefox/85.0",
    }

    def __init__(
        self,
        auto_compress: bool = False,
        add_watermark: bool = False,
        ignore_cache: bool = False,
    ):
        super().__init__(auto_compress, add_watermark, ignore_cache)

        self.cookie: Optional[str] = None
        self.token: Optional[str] = None
        if self.auth_info:
            self.cookie = self.auth_info["cookie"]
            self.token = self.auth_info["token"]
            self.username = self.auth_info["username"]
            self.__headers["Cookie"] = self.cookie

    def login(self, username: str, password: str) -> bool:
        url = self._url("login")
        auth_token, cookie = self._parse_auth_token()

        if not auth_token or not cookie:
            raise MissingAuth("auth token or cookie is None")

        headers = self.__headers.copy()
        headers["Cookie"] = cookie

        data = {
            "login-subject": username,
            "password": password,
            "auth_token": auth_token,
        }

        resp = requests.post(
            url, headers=headers, data=data, allow_redirects=False, timeout=self.timeout
        )
        if resp.status_code == 301:
            # If there is a KEEPLOGIN field in the cookie,
            # pictures will be uploaded as a normal user
            self.cookie = cookie + "; " + resp.headers["Set-Cookie"].split("; ")[0]

            # If there is no KEEPLOGIN field in the cookie,
            # pictures will be uploaded as a tourist
            # self.cookie = cookie

            auth_info = {
                "token": auth_token,
                "cookie": self.cookie,
                "username": username,
                "password": password,
            }
            self._save_auth_info(auth_info)
            return True

        return False

    def _parse_auth_token(self) -> Tuple[Optional[str], Optional[str]]:
        url = self._url("login")
        resp = requests.get(url, headers=self.__headers, timeout=self.timeout)
        if resp.status_code == 200:
            auth_token = re.search(
                r'PF.obj.config.auth_token = "([a-f0-9]{40})"', resp.text
            )
            if not auth_token:
                return None, None
            resp_set_cookie = resp.headers["Set-Cookie"].split("; ")[0]
            return auth_token.group(1), resp_set_cookie
        else:
            logger.error("response error: status_code=%d", resp.status_code)
            return None, None

    @property
    def headers(self):
        assert self.cookie != None

        headers = self.__headers.copy()
        headers.update(
            {
                "Cookie": self.cookie,
            }
        )
        return headers

    def _update_auth_token(self):
        resp = requests.get(self.base_url, headers=self.headers, timeout=self.timeout)
        auth_token = re.search(
            r'PF.obj.config.auth_token = "([a-f0-9]{40})"', resp.text
        )

        assert auth_token != None

        auth_token = auth_token.group(1)
        if not self.auth_info:
            self.auth_info = {}

        self.auth_info["token"] = self.token = auth_token
        self._save_auth_info(self.auth_info)

    def __upload(self, image: ImageType, retries=0) -> Union[str, UploadErrorResponse]:
        url, md5, ok = self._check_cache(image)  # type: ignore

        if ok and not self.ignore_cache:
            return url

        self.check_login()

        image = self._compress_image(image)

        if isinstance(image, Path):
            image = self._add_watermark(image)

        url = self._url("json")
        filename_with_suffix = os.path.basename(str(image))
        filename_without_suffix, suffix = os.path.splitext(filename_with_suffix)
        if isinstance(image, Path):
            if suffix.lower() == ".apng":
                suffix = ".png"
            filename = filename_without_suffix + suffix
        else:
            filename = filename_with_suffix + "." + image.mime_type

        mime_type = (
            mimetypes.guess_type(image)[0]
            if isinstance(image, Path)
            else image.mime_type
        )

        timestamp = int(time.time() * 1000)

        if isinstance(image, Path):
            with open(image, "rb") as fb:
                img_buffer = fb.read()
        else:
            img_buffer = image.stream

        data = {
            "type": "file",
            "action": "upload",
            "timestamp": str(timestamp),
            "auth_token": self.token,
            "nsfw": "0",
        }

        files = {
            "source": (filename, img_buffer, mime_type),
        }

        resp = requests.post(
            url, headers=self.headers, data=data, files=files, timeout=self.timeout  # type: ignore
        )
        resp.encoding = "utf-8"

        try:
            json_resp = resp.json()
        except json.decoder.JSONDecodeError:
            return UploadErrorResponse(resp.status_code, resp.text, str(image))

        try:
            uploaded_url: str = json_resp["image"]["image"]["url"]
            logger.info(
                "uploaded url: '%s' => '%s'",
                image,
                uploaded_url,
            )

            self.cache.save(
                md5,
                IMAGE_BEDS_NAME[self.image_bed_code],
                uploaded_url,
                self.ignore_cache,
            )

            return uploaded_url
        except KeyError:
            if json_resp["error"]["message"] == "请求被拒绝 (auth_token)":
                if retries >= 3:
                    return UploadErrorResponse(
                        resp.status_code, resp.json()["error"]["message"], str(image)
                    )

                logger.warn(
                    "`auth_token` has expired, the program will try to update `auth_token` automatically, number of retries: %d",
                    retries + 1,
                )
                self._update_auth_token()

                return self.__upload(image, retries + 1)
            else:
                return UploadErrorResponse(
                    resp.status_code, resp.json()["error"]["message"], str(image)
                )

    def upload_image(self, image_path: ImagePath) -> Union[str, UploadErrorResponse]:
        logger.debug("uploading: %s", image_path)

        return self.__upload(image_path)

    def upload_image_stream(
        self, image: ImageStream
    ) -> Union[str, UploadErrorResponse]:
        logger.debug("uploading: %s", image.filename)

        return self.__upload(image)

    def get_all_images(self) -> Union[List[ImgtuResponse], ErrorResponse]:
        self.check_login()

        url = self._url(self.username)

        # 集合去重
        images = set()

        def visit_next_page(url):
            resp = requests.get(url, headers=self.__headers, timeout=self.timeout)
            resp.encoding = "utf-8"

            if resp.status_code != 200:
                return ErrorResponse(resp.status_code, resp.text)

            # 只获取默认的公开相册里的图片
            # images_object = re.findall(
            #     r"data-privacy=\"public\" data-object='(.+?)'", resp.text)
            # 获取用户所有图片
            images_object = re.findall(r"data-object='(.+?)'", resp.text)
            images_object = [json.loads(parse.unquote(i)) for i in images_object]
            for image in images_object:
                images.add(
                    (
                        image["url"],
                        image["display_url"],
                        image["id_encoded"],
                        image["width"],
                        image["height"],
                    )
                )

            next_page_url = re.search(
                r'data-pagination="next" href="(.+?)" ><span', resp.text
            )
            if next_page_url:
                next_page_url = next_page_url.group(1)
            else:
                next_page_url = ""

            if next_page_url:
                visit_next_page(next_page_url)

        resp = visit_next_page(url)
        if resp:
            return resp

        result: List[ImgtuResponse] = []

        for item in images:
            result.append(
                ImgtuResponse(
                    item[2],
                    item[0],
                    item[1],
                    int(item[3]),
                    int(item[4]),
                )
            )

        return result

    def delete_image(self, img_id: str, retries=0) -> Optional[ErrorResponse]:
        self.check_login()

        url = self._url("json")
        data = {
            "auth_token": self.token,
            "action": "delete",
            "single": "true",
            "delete": "image",
            "deleting[id]": img_id,
        }
        resp = requests.post(
            url, headers=self.__headers, data=data, timeout=self.timeout
        )
        if resp.status_code == 400:
            if resp.json()["error"]["message"] == "请求被拒绝 (auth_token)":
                if retries >= 3:
                    return ErrorResponse(
                        resp.status_code, resp.json()["error"]["message"]
                    )

                logger.warn(
                    "`auth_token` has expired, the program will try to update `auth_token` automatically, number of retries: %d",
                    retries + 1,
                )
                self._update_auth_token()
                return self.delete_image(img_id, retries + 1)

        if resp.status_code == 200:
            return None

        return ErrorResponse(resp.status_code, resp.json()["error"]["message"])

    def delete_images(self, imgs_id: List[str], retries=0) -> Dict[str, ErrorResponse]:
        self.check_login()

        url = self._url("json")
        data = {
            "auth_token": self.token,
            "action": "delete",
            "from": "list",
            "multiple": "true",
            "delete": "images",
            "deleting[ids][]": imgs_id,
        }
        resp = requests.post(
            url, headers=self.__headers, data=data, timeout=self.timeout
        )
        json_resp = resp.json()
        if resp.status_code == 400:
            if json_resp["error"]["message"] == "请求被拒绝 (auth_token)":
                if retries >= 3:
                    # 删除多张图片时如果重试3次仍无法成功响应则退出程序
                    logger.fatal(
                        "authentication information is invalid: %s",
                        resp.json()["error"]["message"],
                    )

                logger.warn(
                    "`auth_token` has expired, the program will try to update `auth_token` automatically, number of retries: %d",
                    retries + 1,
                )
                self._update_auth_token()
                return self.delete_images(imgs_id, retries + 1)
            elif json_resp["error"]["code"] == 106:
                # imgtu 只有删除的所有 id 都无效时才会返回这个错误
                # 只要有一个有效 id，就会返回 200
                return {img_id: ErrorResponse(404, "服务器中无此图片") for img_id in imgs_id}

        return {}

    def _url(self, path: str) -> str:
        return self.base_url + path

    def __repr__(self):
        return "imgtu.com"

    @property
    def description(self):
        return "中国大陆访问可能不太稳定"



#!/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Author:      thepoy
# @Email:       thepoy@163.com
# @File Name:   watermark.py
# @Created At:  2021-02-09 15:17:32
# @Modified At: 2023-02-21 12:40:20
# @Modified By: thepoy

import os
from pathlib import Path
from up2b.up2b_lib.custom_types import ImagePath

from up2b.up2b_lib.log import child_logger

logger = child_logger(__name__)

try:
    from PIL import Image, ImageDraw, ImageFont, ImageEnhance
    from PIL.ImageDraw import ImageDraw as Draw
except ModuleNotFoundError:
    logger.fatal(
        "you have enabled the feature of adding watermark, but [ pillow ] is not installed, please execute `pip install pillow` before enabling this feature"
    )

from typing import List, Tuple
from collections import namedtuple

from up2b.up2b_lib.constants import CACHE_PATH
from up2b.up2b_lib.utils import is_ascii

TypeFont = namedtuple("TypeFont", ["text", "size", "font_path", "color"])


class AddWatermark:
    """添加水印的类

    可以向一张或多张图片添加文字或图片水印
    """

    def __init__(self, x: int, y: int, opacity: int = 50):
        """初始化函数

        初始化时，可以传入水印位置、输入路径和透明度

        Arguments:
            position {Tuple[int, int]} -- 水印的位置

        Keyword Arguments:
            output {str} -- 添加水印的图片的保存路径 (default: {"output/"})
            opacity {int} -- 透明度，整型，范围在 [0, 100] ，代表百分比 (default: {50})
        """
        self.x = x
        self.y = y
        self._step = self.y
        assert opacity >= 0 and opacity <= 100, "透明度只能在 0 到 100 之间"
        self.opacity = opacity

    def __set_opacity(self, img: Image.Image) -> Image.Image:
        """设置水印透明度

        Arguments:
            img {Image.Image} -- 要处理的图片对象

        Returns:
            Image.Image -- 已处理好的透明的img
        """
        alpha = img.split()[3]
        alpha = ImageEnhance.Brightness(alpha).enhance(self.opacity / 100)
        img.putalpha(alpha)
        return img

    def __text_watermark_position_handler(
        self, offset: float, size: int, img: Image.Image
    ) -> Tuple[float, int]:
        """设置文字水印位置

        水印坐标(x, y)（水印的参考点为左上角）
            当x, y >= 0 时，水印坐标从左上角按x、y的值移动到指定位置，
            当x < 0 且 y >= 0 时，水印从右上角移动，
            当x >= 0 且 y < 0 时，水印从右下角移动，
            当x，y < 0时，水印从左下角移动。

        Arguments:
            count {int} -- 水印文字的数量
            size {int} -- 字体大小
            img {Image.Image} -- 需要添加水印的图片对象

        Returns:
            Tuple[int, int] -- 计算后的真实坐标
        """
        width, height = img.size
        x, y = self.x, self.y

        if x < 0:
            x = x + width - offset
        if y < 0:
            y = y + height - size
        return x, y

    def __image_watermark_postion_handler(
        self, img: Image.Image, mark: Image.Image
    ) -> Tuple[int, int]:
        """设置图片水印的位置

        水印坐标(x, y)（水印的参考点为左上角）
            当x, y >= 0 时，水印坐标从左上角按x、y的值移动到指定位置，
            当x < 0 且 y >= 0 时，水印从右上角移动，
            当x >= 0 且 y < 0 时，水印从右下角移动，
            当x，y < 0时，水印从左下角移动。

        Arguments:
            img {Image.Image} -- 需添加水印的图片对象
            mark {Image.Image} -- 图片水印对象

        Returns:
            Tuple[int, int] -- 计算后的真实坐标
        """
        x, y = self.x, self.y
        width, height = img.size
        mark_w, mark_h = mark.size

        if x < 0:
            x = x + width - mark_w
            self.x = x
        if y < 0:
            y = y + height - mark_h
            self.y = y
        return x, y

    def _cal_x_offset(self, text: str, size: int) -> float:
        offset = 0.0
        for c in text:
            if is_ascii(c):
                offset += size / 2
            else:
                offset += size
        return offset

    def __convert_image_to_rgba(self, image_path: ImagePath) -> Image.Image:
        img = Image.open(image_path)
        return img.convert("RGBA")

    def __draw_text(self, img: Image.Image, draw: Draw, tf: TypeFont):
        offset = self._cal_x_offset(tf.text, tf.size)
        position = self.__text_watermark_position_handler(offset, tf.size, img)
        if not tf.font_path:
            raise ValueError("font path is required")
        ftf = ImageFont.truetype(tf.font_path, size=tf.size)
        draw.text(position, tf.text, font=ftf, fill=tf.color)  # 将文字填充到字体图层

    def add_text_watermark(
        self,
        image_path: ImagePath,
        texts: List[TypeFont],
    ) -> Path:
        img = self.__convert_image_to_rgba(image_path)

        watermark = Image.new("RGBA", img.size, (0, 0, 0, 0))  # 字体图层

        draw = ImageDraw.Draw(watermark)  # 创建文字水印编辑对象
        texts.reverse()
        for text in texts:
            self.__draw_text(img, draw, text)
            self.y += self._step
        watermark = self.__set_opacity(watermark)  # 设置水印透明度
        del draw  # 删除文字水印编辑对象

        combined = Image.alpha_composite(img, watermark)  # 将原图和字体图层合并
        combined = combined.convert("RGB")
        filename = os.path.splitext(os.path.basename(image_path))[0] + ".jpg"

        if not CACHE_PATH.exists():
            CACHE_PATH.mkdir()

        new_path = CACHE_PATH / filename

        combined.save(new_path)

        return new_path

    def add_image_watermark(
        self,
        image_path: Path,
        image_watermark_path: Path,
        resize: int = 100,
    ):
        """添加图片水印

        添加指定位置、透明度、输出路径、缩放比例和图片水印

        Arguments:
            image_path {str} -- 要添加水印的图片路径
            image_watermark_path {str} -- 要添加和水印路径

        Keyword Arguments:
            resize {int} -- 缩放比例，范围在 [0, 100] 之间的整数，是一个百分比，默认值为100，即为不缩放 (default: {100})
            suffix {str} -- 添加水印图片后的文件名后缀 (default: {"_with_image_watermark"})
        """
        img = self.__convert_image_to_rgba(image_path)
        mark: Image.Image = Image.open(image_watermark_path)

        mark_w, mark_h = mark.size
        mark = mark.resize((int(mark_w * resize / 100), int(mark_h * resize / 100)))

        position = self.__image_watermark_postion_handler(img, mark)

        watermark = Image.new("RGBA", img.size, (0, 0, 0, 0))
        watermark.paste(mark, position)
        watermark = self.__set_opacity(watermark)

        combined = Image.alpha_composite(img, watermark)
        combined = combined.convert("RGB")
        filename = os.path.splitext(os.path.basename(image_path))[0] + ".jpg"

        combined.save(os.path.join(CACHE_PATH, filename))
